# -*- coding: utf-8 -*-
"""pyrandwalk errors."""


class pyrandwalkStateError(Exception):
    """State error class."""

    pass


class pyrandwalkTransitionsError(Exception):
    """Transitions matrix error class."""

    pass


class pyrandwalkVectorError(Exception):
    """Vector error class."""

    pass
